# Concept: Namespaces

Namespaces in C++ provide a mechanism to group related code elements (functions, classes, variables) under a named scope, helping to avoid naming conflicts and organize code logically.

## Key Characteristics

- Prevent name collisions in large projects
- Group related functionality
- Can be defined across multiple files
- Support nesting (namespaces within namespaces)
- Do not affect program performance or memory usage

## Example 1: Basic Namespace Usage

```cpp
#include <iostream>

namespace Mathematics {
    const double PI = 3.14159;
    
    double calculateCircleArea(double radius) {
        return PI * radius * radius;
    }
}

int main() {
    double radius = 5.0;
    std::cout << "Area of circle with radius " << radius << " is: "
              << Mathematics::calculateCircleArea(radius) << std::endl;
    return 0;
}
```

### Explanation

- Demonstrates creation of a simple namespace `Mathematics`
- Shows how to define constants and functions within a namespace
- Illustrates accessing namespace members using the scope resolution operator `::`

## Example 2: Using Directives and Declarations

```cpp
#include <iostream>

namespace Physics {
    const double G = 9.81;  // Acceleration due to gravity
    
    double calculateFallingDistance(double time) {
        return 0.5 * G * time * time;
    }
}

using namespace Physics;
using std::cout;
using std::endl;

int main() {
    double time = 3.0;  // seconds
    cout << "Distance fallen in " << time << " seconds: "
         << calculateFallingDistance(time) << " meters" << endl;
    return 0;
}
```

### Explanation

- Shows use of `using namespace` directive to bring all names from `Physics` into current scope
- Demonstrates `using` declarations for specific names from the `std` namespace
- Illustrates how namespaces can be used to organize domain-specific code (physics calculations in this case)

## Example 3: Nested Namespaces and Anonymous Namespaces

```cpp
#include <iostream>

namespace Science {
    namespace Chemistry {
        double calculateMolarity(double moles, double liters) {
            return moles / liters;
        }
    }
}

namespace {
    // This function is only accessible within this translation unit
    void printResult(double molarity) {
        std::cout << "The molarity is " << molarity << " mol/L" << std::endl;
    }
}

int main() {
    double moles = 0.5;
    double liters = 2.0;
    double molarity = Science::Chemistry::calculateMolarity(moles, liters);
    printResult(molarity);
    return 0;
}
```

### Explanation

- Demonstrates nested namespaces with `Science::Chemistry`
- Shows use of an anonymous namespace for internal linkage
- Illustrates how namespaces can be used to create hierarchical organization of code

## Additional Considerations

- Inline namespaces (C++11) can be used for versioning
- Namespace aliases can be created for long namespace names
- The `std` namespace contains all standard library components

## Summary

Namespaces in C++ provide a powerful mechanism for organizing code and preventing naming conflicts. They allow developers to group related functionality, create hierarchical code structures, and manage the scope of identifiers. Through the use of namespace directives, declarations, and the scope resolution operator, C++ programmers can control how names from different namespaces are used in their code. Proper use of namespaces is crucial in large-scale software development and when working with multiple libraries.
